/* Copyright 2012, Sebastian Reichel <sre@ring0.de>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

public class ADXL345 : I2CDevice, Accelerometer {
	public ADXL345(uint8 dev, uint8 addr = 0x53) throws I2CError {
			setup(dev, addr);

			set_byte(0x2D, 0x00); /* reset */
			set_byte(0x31, 0x01); /* +/- 4g */
			set_byte(0x2C, 0x0b); /* 200hz */
			set_byte(0x2D, 0x18); /* mode: measurement + auto sleep */
	}

	public void init(KeyFile cfg) throws KeyFileError, I2CError {
		var adapter = cfg.get_uint64("ADXL345", "i2c-adapter");
		var address = cfg.get_uint64("ADXL345", "i2c-address");
		setup((uint8) adapter, (uint8) address);
	}

	public uint8 get_address() throws I2CError {
		var addr = get_byte(0x00);
		assert(addr == 0xe5);
		return addr;
	}

	public void get_offset(out uint8 x, out uint8 y, out uint8 z) throws I2CError {
		x = get_byte(0x1E);
		y = get_byte(0x1F);
		z = get_byte(0x20);
	}

	public void set_offset(uint8 x, uint8 y, uint8 z) throws I2CError {
		set_byte(0x1E, x);
		set_byte(0x1F, y);
		set_byte(0x20, z);
	}

	public void get_data(out int16 x, out int16 y, out int16 z) throws I2CError {
		var data = get_block(0x32, 6);
		x = data[0] | data[1] << 8;
		y = data[2] | data[3] << 8;
		z = data[4] | data[5] << 8;
	}
}

public Type register_plugin(Module module) {
	// types are registered automatically
	return typeof(ADXL345);
}
